@@ -6,4 +6,7 @@ is contained within a subpackage of the pysat.instruments
 package.
 """
 
-__all__ = ['icon']
+__all__ = ['testing', 'testing_empty', 'single_test']
+
+for const in __all__:
+    exec("from pysat.constellations import {:}".format(const))
