@@ -5,4 +5,5 @@ specific functions for instruments and classes of instruments.
 Each set of methods is contained within a subpackage of this set.
 """
 
-from . import demeter, general, icon, madrigal, nasa_cdaweb, sw, testing
+from pysat.instruments.methods import general
+from pysat.instruments.methods import testing
